INSERT INTO pipeline.anomalies (label) VALUES
    ('TM_DPU_TC_ACC_ILLEGAL_APID'),
    ('TM_DPU_TC_ACC_INCOMPLETE'),
    ('TM_DPU_TC_ACC_WRONG_CRC'),
    ('TM_DPU_TC_ACC_ILLEGAL_TYPE'),
    ('TM_DPU_TC_ACC_ILLEGAL_SUBTYPE'),
    ('TM_DPU_TC_ACC_NOT_ALLOWED'),
    ('TM_DPU_TC_ACC_WRONG_SOURCE_ID'),
    ('TM_DPU_TC_EXE_INCONSISTENT'),
    ('TM_DPU_TC_EXE_NOT_EXECUTABLE'),
    ('TM_DPU_TC_EXE_NOT_IMPLEMENTED'),
    ('TM_DPU_TC_EXE_ERROR'),
    ('TM_DPU_OBC_TC_ACC_ILLEGAL_APID'),
    ('TM_DPU_OBC_TC_ACC_INCOMPLETE'),
    ('TM_DPU_OBC_TC_ACC_WRONG_CRC'),
    ('TM_DPU_OBC_TC_ACC_ILLEGAL_TYPE'),
    ('TM_DPU_OBC_TC_ACC_ILLEGAL_SUBTYPE'),
    ('TM_DPU_OBC_TC_ACC_NOT_ALLOWED'),
    ('TM_DPU_OBC_TC_ACC_WRONG_SOURCE_ID'),
    ('TM_DPU_OBC_TC_EXE_INCONSISTENT'),
    ('TM_DPU_OBC_TC_EXE_NOT_EXECUTABLE'),
    ('TM_DPU_OBC_TC_EXE_NOT_IMPLEMENTED'),
    ('TM_DPU_OBC_TC_EXE_ERROR'),
    ('TM_DPU_EVENT_LE_DPU_ADC'),
    ('TM_DPU_EVENT_LE_DPU_PDU_LINK'),
    ('TM_DPU_EVENT_LE_DPU_SC_LINK'),
    ('TM_DPU_EVENT_LE_DPU_PDU_SIS'),
    ('TM_DPU_EVENT_LE_DPU_AHB'),
    ('TM_DPU_EVENT_LE_DPU_SC_SPW'),
    ('TM_DPU_EVENT_LE_DPU_TIME'),
    ('TM_DPU_EVENT_LE_DPU_TIMECODE'),
    ('TM_DPU_EVENT_LE_DPU_RESET'),
    ('TM_DPU_EVENT_ME_DPU_AHB'),
    ('TM_DPU_EVENT_ME_DPU_EEPROM'),
    ('TM_DPU_EVENT_ME_DPU_MEMORY'),
    ('TM_DPU_EVENT_ME_DPU_PDU_SIS'),
    ('TM_DPU_EVENT_ME_DPU_SC_SPW'),
    ('TM_DPU_EVENT_ME_DPU_BOOT'),
    ('TM_DPU_EVENT_ME_DBS_BUFFER'),
    ('TM_DPU_EVENT_ME_DPU_PARAM'),
    ('TM_DPU_EVENT_HE_DPU_ADC'),
    ('TM_DPU_EVENT_HE_DPU_PDU_LINK'),
    ('TM_DPU_EVENT_HE_DPU_RESET'),
    ('TM_DPU_PID5_TC_ACC_INCOMPLETE'),
    ('TM_DPU_PID5_TC_ACC_WRONG_CRC'),
    ('TM_DPU_PID5_TC_ACC_ILLEGAL_TYPE'),
    ('TM_DPU_PID5_TC_ACC_ILLEGAL_SUBTYPE'),
    ('TM_DPU_PID5_TC_ACC_NOT_ALLOWED'),
    ('TM_DPU_PID5_TC_ACC_WRONG_SOURCE_ID'),
    ('TM_DPU_PID5_TC_ACC_ILLEGAL_APID'),
    ('TM_DPU_PID5_TC_EXE_INCONSISTENT'),
    ('TM_DPU_PID5_TC_EXE_NOT_EXECUTABLE'),
    ('TM_DPU_PID5_TC_EXE_NOT_IMPLEMENTED'),
    ('TM_DPU_PID5_TC_EXE_ERROR'),
    ('TM_DPU_PID6_TC_ACC_INCOMPLETE'),
    ('TM_DPU_PID6_TC_ACC_WRONG_CRC'),
    ('TM_DPU_PID6_TC_ACC_ILLEGAL_TYPE'),
    ('TM_DPU_PID6_TC_ACC_ILLEGAL_SUBTYPE'),
    ('TM_DPU_PID6_TC_ACC_NOT_ALLOWED'),
    ('TM_DPU_PID6_TC_ACC_WRONG_SOURCE_ID'),
    ('TM_DPU_PID6_TC_ACC_ILLEGAL_APID'),
    ('TM_DPU_PID6_TC_EXE_INCONSISTENT'),
    ('TM_DPU_PID6_TC_EXE_NOT_EXECUTABLE'),
    ('TM_DPU_PID6_TC_EXE_NOT_IMPLEMENTED'),
    ('TM_DPU_PID6_TC_EXE_ERROR'),
    ('TM_DPU_EVENT_LE_DPU_LFR_SPW'),
    ('TM_DPU_EVENT_LE_DPU_TDS_SPW'),
    ('TM_DPU_EVENT_LE_DPU_THR_SPW'),
    ('TM_DPU_EVENT_LE_DPU_LFR_RMAP'),
    ('TM_DPU_EVENT_LE_DPU_TDS_RMAP'),
    ('TM_DPU_EVENT_LE_DPU_THR_RMAP'),
    ('TM_DPU_EVENT_LE_RPW_CURRENT'),
    ('TM_DPU_EVENT_LE_DPU_BIA_SIS'),
    ('TM_DPU_EVENT_LE_DPU_BIA_LINK'),
    ('TM_DPU_EVENT_LE_DPU_COMP'),
    ('TM_DPU_EVENT_LE_DPU_LFR_LINK'),
    ('TM_DPU_EVENT_LE_DPU_TDS_LINK'),
    ('TM_DPU_EVENT_LE_DPU_THR_LINK'),
    ('TM_DPU_EVENT_ME_DPU_LFR_SPW'),
    ('TM_DPU_EVENT_ME_DPU_TDS_SPW'),
    ('TM_DPU_EVENT_ME_DPU_THR_SPW'),
    ('TM_DPU_EVENT_ME_DPU_LFR_RMAP'),
    ('TM_DPU_EVENT_ME_DPU_TDS_RMAP'),
    ('TM_DPU_EVENT_ME_DPU_THR_RMAP'),
    ('TM_DPU_EVENT_ME_RPW_CURRENT'),
    ('TM_DPU_EVENT_ME_LFR_BOOT'),
    ('TM_DPU_EVENT_ME_TDS_BOOT'),
    ('TM_DPU_EVENT_ME_THR_BOOT'),
    ('TM_DPU_EVENT_ME_DPU_BIA_SIS'),
    ('TM_DPU_EVENT_ME_DAS_BUFFER'),
    ('TM_DPU_EVENT_ME_LFR_PROCESSOR'),
    ('TM_DPU_EVENT_ME_TDS_PROCESSOR'),
    ('TM_DPU_EVENT_ME_THR_PROCESSOR'),
    ('TM_DPU_EVENT_ME_LFR_COMMAND'),
    ('TM_DPU_EVENT_ME_TDS_COMMAND'),
    ('TM_DPU_EVENT_ME_THR_COMMAND'),
    ('TM_DPU_EVENT_ME_DPU_MODE'),
    ('TM_DPU_EVENT_ME_LFR_MODE'),
    ('TM_DPU_EVENT_ME_TDS_MODE'),
    ('TM_DPU_EVENT_ME_THR_MODE'),
    ('TM_DPU_EVENT_ME_PDU_OPERATION'),
    ('TM_DPU_EVENT_ME_DPU_TIME'),
    ('TM_DPU_EVENT_ME_DPU_SC_LINK'),
    ('TM_DPU_EVENT_ME_BIA_SWEEP'),
    ('TM_DPU_EVENT_ME_BIA_CALIB'),
    ('TM_DPU_EVENT_ME_DPU_WAIT_SYNC'),
    ('TM_DPU_EVENT_HE_RPW_TEMPERATURE'),
    ('TM_DPU_EVENT_HE_DPU_BIA_LINK'),
    ('TM_DPU_EVENT_HE_DPU_LFR_LINK'),
    ('TM_DPU_EVENT_HE_DPU_TDS_LINK'),
    ('TM_DPU_EVENT_HE_DPU_THR_LINK'),
    ('TM_DPU_EVENT_HE_BIAS_HV'),
    ('TM_DPU_EVENT_HE_RPW_OVC'),
    ('TM_DPU_EVENT_HE_RPW_OVV'),
    ('TM_LFR_TC_ACC_INCOMPLETE'),
    ('TM_LFR_TC_ACC_WRONG_CRC'),
    ('TM_LFR_TC_ACC_ILLEGAL_TYPE'),
    ('TM_LFR_TC_ACC_ILLEGAL_SUBTYPE'),
    ('TM_LFR_TC_ACC_NOT_ALLOWED'),
    ('TM_LFR_TC_ACC_WRONG_SOURCE_ID'),
    ('TM_LFR_TC_ACC_ILLEGAL_APID'),
    ('TM_LFR_TC_EXE_INCONSISTENT'),
    ('TM_LFR_TC_EXE_NOT_EXECUTABLE'),
    ('TM_LFR_TC_EXE_NOT_IMPLEMENTED'),
    ('TM_LFR_TC_EXE_ERROR'),
    ('TM_LFR_TC_EXE_CORRUPTED'),
    ('TM_TDS_TC_ACC_INCOMPLETE'),
    ('TM_TDS_TC_ACC_WRONG_CRC'),
    ('TM_TDS_TC_ACC_ILLEGAL_TYPE'),
    ('TM_TDS_TC_ACC_ILLEGAL_SUBTYPE'),
    ('TM_TDS_TC_ACC_NOT_ALLOWED'),
    ('TM_TDS_TC_ACC_WRONG_SOURCE_ID'),
    ('TM_TDS_TC_ACC_ILLEGAL_APID'),
    ('TM_TDS_TC_EXE_INCONSISTENT'),
    ('TM_TDS_TC_EXE_NOT_EXECUTABLE'),
    ('TM_TDS_TC_EXE_NOT_IMPLEMENTED'),
    ('TM_TDS_TC_EXE_ERROR'),
    ('TM_TDS_TC_EXE_CORRUPTED'),
    ('TM_THR_TC_ACC_INCOMPLETE'),
    ('TM_THR_TC_ACC_WRONG_CRC'),
    ('TM_THR_TC_ACC_ILLEGAL_TYPE'),
    ('TM_THR_TC_ACC_ILLEGAL_SUBTYPE'),
    ('TM_THR_TC_ACC_NOT_ALLOWED'),
    ('TM_THR_TC_ACC_WRONG_SOURCE_ID'),
    ('TM_THR_TC_ACC_ILLEGAL_APID'),
    ('TM_THR_TC_EXE_INCONSISTENT'),
    ('TM_THR_TC_EXE_NOT_EXECUTABLE'),
    ('TM_THR_TC_EXE_NOT_IMPLEMENTED'),
    ('TM_THR_TC_EXE_ERROR'),
    ('TM_THR_TC_EXE_CORRUPTED'),
    ('TM_RPW_TC_ACC_ILLEGAL_APID'),
    ('TM_RPW_TC_ACC_INCOMPLETE'),
    ('TM_RPW_TC_ACC_WRONG_CRC'),
    ('TM_RPW_TC_ACC_ILLEGAL_TYPE'),
    ('TM_RPW_TC_ACC_ILLEGAL_SUBTYPE'),
    ('TM_RPW_TC_ACC_NOT_ALLOWED'),
    ('TM_RPW_TC_ACC_WRONG_SOURCE_ID'),
    ('TM_RPW_TC_EXE_INCONSISTENT'),
    ('TM_RPW_TC_EXE_NOT_EXECUTABLE'),
    ('TM_RPW_TC_EXE_NOT_IMPLEMENTED'),
    ('TM_RPW_TC_EXE_ERROR'),
    ('TM_RPW_TC_EXE_CORRUPTED');

UPDATE pipeline.anomalies set tracked = false where label = 'TM_DPU_EVENT_LE_DPU_AHB';
