-- FROM ComponentEFECSTypes.xsd

INSERT INTO pipeline.events (label, origin) VALUES
    ('TCM', 'SOLO'),
    ('HGA_PT', 'SOLO'),
    ('MGA_PT', 'SOLO'),
    ('WOL', 'SOLO'),
    ('SA_ROT', 'SOLO'),
    ('ROLL', 'SOLO'),
    ('CALIB_ROLL_MAG', 'SOLO'),
    ('CALIB_ROLL_RPW', 'SOLO'),
    ('CALIB_ROLL_RS', 'SOLO'),
    ('CALIB_OFFPOI_FLATFIELD', 'SOLO'),
    ('CALIB_OFFPOI_STRAYLIGHT', 'SOLO'),
    ('CALIB_OFFPOI_ALIGNMENT', 'SOLO'),
    ('CALIB_OFFPOI_STAR', 'SOLO'),
    ('CALIB_OFFPOI_OOF', 'SOLO'),
    ('PTR_START', 'SOLO'),
    ('PTR_END', 'SOLO'),
    ('VSTP_UPDATE', 'SOLO'),
    ('MAINT', 'SOLO'),
    ('NEAR_SUN', 'SOLO'),
    ('FAR_SUN', 'SOLO'),
    ('SWA_TCS_NEAR', 'SOLO'),
    ('SWA_TCS_FAR', 'SOLO'),
    ('NAV', 'SOLO'),
    ('STP_START', 'SOLO'),
    ('SLEW', 'SOLO'),
    ('SSMM_DUMP_1WAY', 'SOLO'),
    ('SSMM_DUMP_2WAY', 'SOLO'),
    ('OMM_DUMP', 'SOLO'),
    ('PASS', 'SOLO'),
    ('RSW_START', 'SOLO'),
    ('RSW_END', 'SOLO'),
    ('SUP_CONJ_START', 'SOLO'),
    ('SUP_CONJ_END', 'SOLO'),
    ('POWER', 'SOLO'),
    ('LTP_BOUND', 'SOLO'),
    ('con', 'SOLO'),
    ('tra', 'SOLO'),
    ('ENG_WIN', 'SOLO'),
    ('GAM', 'SOLO'),
    ('ATT_DIST', 'SOLO'),
    ('POINT_CENTRE', 'SOLO'),
    ('POINT_POLE', 'SOLO'),
    ('POINT_LIMB', 'SOLO'),
    ('POINT_FEATURE', 'SOLO'),
    ('POINT_PATTERN', 'SOLO'),
    ('POINT_OTHER', 'SOLO'),
    ('EMC_MAND_QUIET', 'SOLO'),
    ('EMC_PREF_NOISY', 'SOLO'),
    ('IS_COORD_BURST', 'SOLO'),
    ('RS_THERMAL_SAFE', 'SOLO'),
    ('SPICE_OU_NEAR', 'SOLO'),
    ('SPICE_OU_FAR', 'SOLO'),
    ('RSW_EXT_EUI', 'SOLO'),
    ('RSW_EXT_METIS', 'SOLO'),
    ('RSW_EXT_PHI', 'SOLO'),
    ('RSW_EXT_SOLOHI', 'SOLO'),
    ('RSW_EXT_SPICE', 'SOLO'),
    ('RSW_EXT_STIX', 'SOLO'),
    ('TAC_EUI', 'SOLO'),
    ('TAC_METIS', 'SOLO'),
    ('TAC_PHI', 'SOLO'),
    ('TAC_SOLOHI', 'SOLO'),
    ('TAC_SPICE', 'SOLO'),
    ('TAC_STIX', 'SOLO'),
    ('TAC_EPD', 'SOLO'),
    ('TAC_MAG', 'SOLO'),
    ('TAC_RPW', 'SOLO'),
    ('TAC_SWA', 'SOLO'),
    ('HSD_EUI_FSI_OPEN', 'SOLO'),
    ('HSD_EUI_FSI_CLOSE', 'SOLO'),
    ('HSD_EUI_HRI_OPEN', 'SOLO'),
    ('HSD_EUI_HRI_CLOSE', 'SOLO'),
    ('HSD_METIS_OPEN', 'SOLO'),
    ('HSD_METIS_CLOSE', 'SOLO'),
    ('HSD_PHI_FDT_OPEN', 'SOLO'),
    ('HSD_PHI_FDT_CLOSE', 'SOLO'),
    ('HSD_PHI_HRT_OPEN', 'SOLO'),
    ('HSD_PHI_HRT_CLOSE', 'SOLO'),
    ('HSD_SPICE_OPEN', 'SOLO'),
    ('HSD_SPICE_CLOSE', 'SOLO'),
    ('BACKUP_SPLIT', 'SOLO'),
    ('PSP_QUAD', 'SOLO'),
    ('PSP_PERI', 'SOLO'),
    ('PSP_RADA', 'SOLO');

-- FROM IDB
INSERT INTO pipeline.events (label, origin) VALUES
    ('TC_DPU_BOOT_DAS', 'RPW'),
    ('TC_DPU_BOOT_LFR', 'RPW'),
    ('TC_DPU_BOOT_TDS', 'RPW'),
    ('TC_DPU_BOOT_THR', 'RPW'),
    ('TC_DPU_CHECK_MEMORY', 'RPW'),
    ('TC_DPU_CLEAR_BIAS_SWEEP', 'RPW'),
    ('TC_DPU_CLEAR_HK', 'RPW'),
    ('TC_DPU_DAS_DUMP_PAR', 'RPW'),
    ('TC_DPU_DAS_LOAD_COMMON_PAR', 'RPW'),
    ('TC_DPU_DBS_DUMP_PAR', 'RPW'),
    ('TC_DPU_DBS_LOAD_COMMON_PAR', 'RPW'),
    ('TC_DPU_DISABLE_COMPRESSION', 'RPW'),
    ('TC_DPU_DISABLE_EVENT', 'RPW'),
    ('TC_DPU_DISABLE_HK', 'RPW'),
    ('TC_DPU_DISABLE_INFO_PKT', 'RPW'),
    ('TC_DPU_DISABLE_SCIENCE', 'RPW'),
    ('TC_DPU_DISABLE_TRANS_MODE', 'RPW'),
    ('TC_DPU_DISABLE_WATCHDOG', 'RPW'),
    ('TC_DPU_DUMP_MEMORY', 'RPW'),
    ('TC_DPU_ENABLE_COMPRESSION', 'RPW'),
    ('TC_DPU_ENABLE_EVENT', 'RPW'),
    ('TC_DPU_ENABLE_HK', 'RPW'),
    ('TC_DPU_ENABLE_INFO_PKT', 'RPW'),
    ('TC_DPU_ENABLE_SCIENCE', 'RPW'),
    ('TC_DPU_ENABLE_TRANS_MODE', 'RPW'),
    ('TC_DPU_ENABLE_WATCHDOG', 'RPW'),
    ('TC_DPU_ENTER_SBM1_DUMP', 'RPW'),
    ('TC_DPU_ENTER_SBM2_ACQ', 'RPW'),
    ('TC_DPU_ENTER_SBM_DETECTION', 'RPW'),
    ('TC_DPU_ENTER_SERVICE', 'RPW'),
    ('TC_DPU_ENTER_STANDBY', 'RPW'),
    ('TC_DPU_ENTER_SURVEY_BACKUP', 'RPW'),
    ('TC_DPU_ENTER_SURVEY_BURST', 'RPW'),
    ('TC_DPU_ENTER_SURVEY_NORMAL', 'RPW'),
    ('TC_DPU_EXECUTE_BIAS_CMD', 'RPW'),
    ('TC_DPU_EXECUTE_PDU_CMD', 'RPW'),
    ('TC_DPU_LOAD_BHV_PAR', 'RPW'),
    ('TC_DPU_LOAD_BIAS_SWEEP', 'RPW'),
    ('TC_DPU_LOAD_MEMORY', 'RPW'),
    ('TC_DPU_LOAD_POWER_PAR', 'RPW'),
    ('TC_DPU_LOAD_RWF_PAR', 'RPW'),
    ('TC_DPU_LOAD_SBM1_PAR', 'RPW'),
    ('TC_DPU_LOAD_SBM2_PAR', 'RPW'),
    ('TC_DPU_LOAD_SUN_DISTANCE', 'RPW'),
    ('TC_DPU_LOAD_TEMP_PAR', 'RPW'),
    ('TC_DPU_LOAD_VSC_PAR', 'RPW'),
    ('TC_DPU_LOAD_WAVEFORM_PAR', 'RPW'),
    ('TC_DPU_OBC_DISABLE_HK', 'RPW'),
    ('TC_DPU_OBC_ENABLE_HK', 'RPW'),
    ('TC_DPU_OBC_UPDATE_HK_PERIOD', 'RPW'),
    ('TC_DPU_PID5_DISABLE_SCIENCE', 'RPW'),
    ('TC_DPU_PID5_ENABLE_SCIENCE', 'RPW'),
    ('TC_DPU_PID6_DISABLE_SCIENCE', 'RPW'),
    ('TC_DPU_PID6_ENABLE_SCIENCE', 'RPW'),
    ('TC_DPU_RESET', 'RPW'),
    ('TC_DPU_RESET_TM_BUFFER', 'RPW'),
    ('TC_DPU_S20_DISABLE_HK', 'RPW'),
    ('TC_DPU_S20_ENABLE_HK', 'RPW'),
    ('TC_DPU_S20_UPDATE_HK_PERIOD', 'RPW'),
    ('TC_DPU_SAVE_CONTEXT', 'RPW'),
    ('TC_DPU_SET_BIAS1', 'RPW'),
    ('TC_DPU_SET_BIAS2', 'RPW'),
    ('TC_DPU_SET_BIAS3', 'RPW'),
    ('TC_DPU_SET_BIAS_MODE', 'RPW'),
    ('TC_DPU_SET_BIAS_PAGE', 'RPW'),
    ('TC_DPU_SET_BIAS_RELAY', 'RPW'),
    ('TC_DPU_SET_BIAS_RFREQ', 'RPW'),
    ('TC_DPU_SET_BIAS_SWEEP', 'RPW'),
    ('TC_DPU_START_BIAS_CALIB', 'RPW'),
    ('TC_DPU_START_BIAS_SWEEP', 'RPW'),
    ('TC_DPU_START_FROM_CONTEXT', 'RPW'),
    ('TC_DPU_START_TEST', 'RPW'),
    ('TC_DPU_STOP_TEST', 'RPW'),
    ('TC_DPU_SWITCH_OFF_EQUIPMENT', 'RPW'),
    ('TC_DPU_SWITCH_ON_EQUIPMENT', 'RPW'),
    ('TC_DPU_TEST_CONNECTION', 'RPW'),
    ('TC_DPU_UPDATE_HK_PERIOD', 'RPW'),
    ('TC_DPU_UPDATE_INFO', 'RPW'),
    ('TC_DPU_UPDATE_TIME', 'RPW'),
    ('TC_LFR_CHECK_MEMORY', 'RPW'),
    ('TC_LFR_DISABLE_CALIBRATION', 'RPW'),
    ('TC_LFR_DISABLE_HK', 'RPW'),
    ('TC_LFR_DISABLE_SCIENCE', 'RPW'),
    ('TC_LFR_DUMP_KCOEFFICIENTS', 'RPW'),
    ('TC_LFR_DUMP_MEMORY', 'RPW'),
    ('TC_LFR_DUMP_PAR', 'RPW'),
    ('TC_LFR_ENABLE_CALIBRATION', 'RPW'),
    ('TC_LFR_ENABLE_HK', 'RPW'),
    ('TC_LFR_ENABLE_SCIENCE', 'RPW'),
    ('TC_LFR_ENTER_MODE', 'RPW'),
    ('TC_LFR_LOAD_BURST_PAR', 'RPW'),
    ('TC_LFR_LOAD_COMMON_PAR', 'RPW'),
    ('TC_LFR_LOAD_FBINS_MASK', 'RPW'),
    ('TC_LFR_LOAD_FILTER_PAR', 'RPW'),
    ('TC_LFR_LOAD_KCOEFFICIENTS', 'RPW'),
    ('TC_LFR_LOAD_MEMORY', 'RPW'),
    ('TC_LFR_LOAD_NORMAL_PAR', 'RPW'),
    ('TC_LFR_LOAD_SBM1_PAR', 'RPW'),
    ('TC_LFR_LOAD_SBM2_PAR', 'RPW'),
    ('TC_LFR_RESET', 'RPW'),
    ('TC_LFR_UPDATE_HK_PERIOD', 'RPW'),
    ('TC_LFR_UPDATE_INFO', 'RPW'),
    ('TC_LFR_UPDATE_TIME', 'RPW'),
    ('TC_RPW_DISABLE_SCIENCE', 'RPW'),
    ('TC_RPW_ENABLE_SCIENCE', 'RPW'),
    ('TC_RPW_ENTER_MODE', 'RPW'),
    ('TC_RPW_UPDATE_INFO', 'RPW'),
    ('TC_TDS_CHECK_MEMORY', 'RPW'),
    ('TC_TDS_DISABLE_HK', 'RPW'),
    ('TC_TDS_DISABLE_SCIENCE', 'RPW'),
    ('TC_TDS_DUMP_MEMORY', 'RPW'),
    ('TC_TDS_DUMP_NORMAL_TSWF', 'RPW'),
    ('TC_TDS_DUMP_PAR', 'RPW'),
    ('TC_TDS_DUMP_SBM2_TSWF', 'RPW'),
    ('TC_TDS_ENABLE_HK', 'RPW'),
    ('TC_TDS_ENABLE_SCIENCE', 'RPW'),
    ('TC_TDS_ENTER_MODE', 'RPW'),
    ('TC_TDS_LOAD_BURST_PAR', 'RPW'),
    ('TC_TDS_LOAD_COMMON_PAR', 'RPW'),
    ('TC_TDS_LOAD_LFM_PAR', 'RPW'),
    ('TC_TDS_LOAD_MEMORY', 'RPW'),
    ('TC_TDS_LOAD_NORMAL_PAR', 'RPW'),
    ('TC_TDS_LOAD_SBM1_PAR', 'RPW'),
    ('TC_TDS_LOAD_SBM2_PAR', 'RPW'),
    ('TC_TDS_PERFORM_DIAGNOSIS', 'RPW'),
    ('TC_TDS_RESET', 'RPW'),
    ('TC_TDS_UPDATE_HK_PERIOD', 'RPW'),
    ('TC_TDS_UPDATE_INFO', 'RPW'),
    ('TC_TDS_UPDATE_TIME', 'RPW'),
    ('TC_THR_CHECK_MEMORY', 'RPW'),
    ('TC_THR_DISABLE_HK', 'RPW'),
    ('TC_THR_DISABLE_SCIENCE', 'RPW'),
    ('TC_THR_DUMP_MEMORY', 'RPW'),
    ('TC_THR_DUMP_PAR', 'RPW'),
    ('TC_THR_ENABLE_CALIBRATION', 'RPW'),
    ('TC_THR_ENABLE_HK', 'RPW'),
    ('TC_THR_ENABLE_SCIENCE', 'RPW'),
    ('TC_THR_ENTER_MODE', 'RPW'),
    ('TC_THR_LOAD_BURST_PAR_1', 'RPW'),
    ('TC_THR_LOAD_BURST_PAR_2', 'RPW'),
    ('TC_THR_LOAD_BURST_PAR_3', 'RPW'),
    ('TC_THR_LOAD_CALIBRATION_PAR', 'RPW'),
    ('TC_THR_LOAD_MEMORY', 'RPW'),
    ('TC_THR_LOAD_NORMAL_PAR_1', 'RPW'),
    ('TC_THR_LOAD_NORMAL_PAR_2', 'RPW'),
    ('TC_THR_LOAD_NORMAL_PAR_3', 'RPW'),
    ('TC_THR_RESET', 'RPW'),
    ('TC_THR_UPDATE_HK_PERIOD', 'RPW'),
    ('TC_THR_UPDATE_INFO', 'RPW'),
    ('TC_THR_UPDATE_TIME', 'RPW'),
    ('TM_DPU_BIA_HK', 'RPW'),
    ('TM_DPU_DAS_HK', 'RPW'),
    ('TM_DPU_DAS_PARAMETER_DUMP', 'RPW'),
    ('TM_DPU_DAS_SBM1_HK', 'RPW'),
    ('TM_DPU_DAS_STATISTICS_HK', 'RPW'),
    ('TM_DPU_DBS_HK', 'RPW'),
    ('TM_DPU_DBS_PARAMETER_DUMP', 'RPW'),
    ('TM_DPU_EVENT_HE_BIAS_HV', 'RPW'),
    ('TM_DPU_EVENT_HE_DPU_ADC', 'RPW'),
    ('TM_DPU_EVENT_HE_DPU_BIA_LINK', 'RPW'),
    ('TM_DPU_EVENT_HE_DPU_LFR_LINK', 'RPW'),
    ('TM_DPU_EVENT_HE_DPU_PDU_LINK', 'RPW'),
    ('TM_DPU_EVENT_HE_DPU_RESET', 'RPW'),
    ('TM_DPU_EVENT_HE_DPU_TDS_LINK', 'RPW'),
    ('TM_DPU_EVENT_HE_DPU_THR_LINK', 'RPW'),
    ('TM_DPU_EVENT_HE_RPW_OVC', 'RPW'),
    ('TM_DPU_EVENT_HE_RPW_OVV', 'RPW'),
    ('TM_DPU_EVENT_HE_RPW_TEMPERATURE', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_ADC', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_AHB', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_BIA_LINK', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_BIA_SIS', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_COMP', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_LFR_LINK', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_LFR_RMAP', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_LFR_SPW', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_PDU_LINK', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_PDU_SIS', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_RESET', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_SC_LINK', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_SC_SPW', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_TDS_LINK', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_TDS_RMAP', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_TDS_SPW', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_THR_LINK', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_THR_RMAP', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_THR_SPW', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_TIME', 'RPW'),
    ('TM_DPU_EVENT_LE_DPU_TIMECODE', 'RPW'),
    ('TM_DPU_EVENT_LE_RPW_CURRENT', 'RPW'),
    ('TM_DPU_EVENT_ME_BIA_CALIB', 'RPW'),
    ('TM_DPU_EVENT_ME_BIA_SWEEP', 'RPW'),
    ('TM_DPU_EVENT_ME_DAS_BUFFER', 'RPW'),
    ('TM_DPU_EVENT_ME_DBS_BUFFER', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_AHB', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_BIA_SIS', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_BOOT', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_EEPROM', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_LFR_RMAP', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_LFR_SPW', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_MEMORY', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_MODE', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_PARAM', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_PDU_SIS', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_SC_LINK', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_SC_SPW', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_TDS_RMAP', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_TDS_SPW', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_THR_RMAP', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_THR_SPW', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_TIME', 'RPW'),
    ('TM_DPU_EVENT_ME_DPU_WAIT_SYNC', 'RPW'),
    ('TM_DPU_EVENT_ME_LFR_BOOT', 'RPW'),
    ('TM_DPU_EVENT_ME_LFR_COMMAND', 'RPW'),
    ('TM_DPU_EVENT_ME_LFR_MODE', 'RPW'),
    ('TM_DPU_EVENT_ME_LFR_PROCESSOR', 'RPW'),
    ('TM_DPU_EVENT_ME_PDU_OPERATION', 'RPW'),
    ('TM_DPU_EVENT_ME_RPW_CURRENT', 'RPW'),
    ('TM_DPU_EVENT_ME_TDS_BOOT', 'RPW'),
    ('TM_DPU_EVENT_ME_TDS_COMMAND', 'RPW'),
    ('TM_DPU_EVENT_ME_TDS_MODE', 'RPW'),
    ('TM_DPU_EVENT_ME_TDS_PROCESSOR', 'RPW'),
    ('TM_DPU_EVENT_ME_THR_BOOT', 'RPW'),
    ('TM_DPU_EVENT_ME_THR_COMMAND', 'RPW'),
    ('TM_DPU_EVENT_ME_THR_MODE', 'RPW'),
    ('TM_DPU_EVENT_ME_THR_PROCESSOR', 'RPW'),
    ('TM_DPU_EVENT_PR_BIA_CALIB', 'RPW'),
    ('TM_DPU_EVENT_PR_BIA_SWEEP', 'RPW'),
    ('TM_DPU_EVENT_PR_DAS_BOOT', 'RPW'),
    ('TM_DPU_EVENT_PR_DBS_BOOT', 'RPW'),
    ('TM_DPU_EVENT_PR_DPU_BIA_LINK', 'RPW'),
    ('TM_DPU_EVENT_PR_DPU_LFR_LINK', 'RPW'),
    ('TM_DPU_EVENT_PR_DPU_MODE', 'RPW'),
    ('TM_DPU_EVENT_PR_DPU_PDU_LINK', 'RPW'),
    ('TM_DPU_EVENT_PR_DPU_RECOVERY', 'RPW'),
    ('TM_DPU_EVENT_PR_DPU_SBM1', 'RPW'),
    ('TM_DPU_EVENT_PR_DPU_SBM2', 'RPW'),
    ('TM_DPU_EVENT_PR_DPU_TDS_LINK', 'RPW'),
    ('TM_DPU_EVENT_PR_DPU_THR_LINK', 'RPW'),
    ('TM_DPU_EVENT_PR_DPU_WAIT_SYNC', 'RPW'),
    ('TM_DPU_EVENT_PR_LFR_BOOT', 'RPW'),
    ('TM_DPU_EVENT_PR_LFR_COMMAND', 'RPW'),
    ('TM_DPU_EVENT_PR_LFR_MODE', 'RPW'),
    ('TM_DPU_EVENT_PR_PDU_OPERATION', 'RPW'),
    ('TM_DPU_EVENT_PR_TDS_BOOT', 'RPW'),
    ('TM_DPU_EVENT_PR_TDS_COMMAND', 'RPW'),
    ('TM_DPU_EVENT_PR_TDS_MODE', 'RPW'),
    ('TM_DPU_EVENT_PR_THR_BOOT', 'RPW'),
    ('TM_DPU_EVENT_PR_THR_COMMAND', 'RPW'),
    ('TM_DPU_EVENT_PR_THR_MODE', 'RPW'),
    ('TM_DPU_IIT_HK', 'RPW'),
    ('TM_DPU_LINK_CONNECTION_REPORT', 'RPW'),
    ('TM_DPU_MEMORY_CHECK', 'RPW'),
    ('TM_DPU_MEMORY_DUMP_EEPROM', 'RPW'),
    ('TM_DPU_MEMORY_DUMP_PROM', 'RPW'),
    ('TM_DPU_MEMORY_DUMP_RAM', 'RPW'),
    ('TM_DPU_MEMORY_DUMP_REGISTER', 'RPW'),
    ('TM_DPU_OBC_HK', 'RPW'),
    ('TM_DPU_OBC_TC_ACC_ILLEGAL_APID', 'RPW'),
    ('TM_DPU_OBC_TC_ACC_ILLEGAL_SUBTYPE', 'RPW'),
    ('TM_DPU_OBC_TC_ACC_ILLEGAL_TYPE', 'RPW'),
    ('TM_DPU_OBC_TC_ACC_INCOMPLETE', 'RPW'),
    ('TM_DPU_OBC_TC_ACC_NOT_ALLOWED', 'RPW'),
    ('TM_DPU_OBC_TC_ACC_SUCCESS', 'RPW'),
    ('TM_DPU_OBC_TC_ACC_WRONG_CRC', 'RPW'),
    ('TM_DPU_OBC_TC_ACC_WRONG_SOURCE_ID', 'RPW'),
    ('TM_DPU_OBC_TC_EXE_ERROR', 'RPW'),
    ('TM_DPU_OBC_TC_EXE_INCONSISTENT', 'RPW'),
    ('TM_DPU_OBC_TC_EXE_NOT_EXECUTABLE', 'RPW'),
    ('TM_DPU_OBC_TC_EXE_NOT_IMPLEMENTED', 'RPW'),
    ('TM_DPU_OBC_TC_EXE_SUCCESS', 'RPW'),
    ('TM_DPU_PDU_HK', 'RPW'),
    ('TM_DPU_PID5_TC_ACC_ILLEGAL_APID', 'RPW'),
    ('TM_DPU_PID5_TC_ACC_ILLEGAL_SUBTYPE', 'RPW'),
    ('TM_DPU_PID5_TC_ACC_ILLEGAL_TYPE', 'RPW'),
    ('TM_DPU_PID5_TC_ACC_INCOMPLETE', 'RPW'),
    ('TM_DPU_PID5_TC_ACC_NOT_ALLOWED', 'RPW'),
    ('TM_DPU_PID5_TC_ACC_SUCCESS', 'RPW'),
    ('TM_DPU_PID5_TC_ACC_WRONG_CRC', 'RPW'),
    ('TM_DPU_PID5_TC_ACC_WRONG_SOURCE_ID', 'RPW'),
    ('TM_DPU_PID5_TC_EXE_ERROR', 'RPW'),
    ('TM_DPU_PID5_TC_EXE_INCONSISTENT', 'RPW'),
    ('TM_DPU_PID5_TC_EXE_NOT_EXECUTABLE', 'RPW'),
    ('TM_DPU_PID5_TC_EXE_NOT_IMPLEMENTED', 'RPW'),
    ('TM_DPU_PID5_TC_EXE_SUCCESS', 'RPW'),
    ('TM_DPU_PID6_TC_ACC_ILLEGAL_APID', 'RPW'),
    ('TM_DPU_PID6_TC_ACC_ILLEGAL_SUBTYPE', 'RPW'),
    ('TM_DPU_PID6_TC_ACC_ILLEGAL_TYPE', 'RPW'),
    ('TM_DPU_PID6_TC_ACC_INCOMPLETE', 'RPW'),
    ('TM_DPU_PID6_TC_ACC_NOT_ALLOWED', 'RPW'),
    ('TM_DPU_PID6_TC_ACC_SUCCESS', 'RPW'),
    ('TM_DPU_PID6_TC_ACC_WRONG_CRC', 'RPW'),
    ('TM_DPU_PID6_TC_ACC_WRONG_SOURCE_ID', 'RPW'),
    ('TM_DPU_PID6_TC_EXE_ERROR', 'RPW'),
    ('TM_DPU_PID6_TC_EXE_INCONSISTENT', 'RPW'),
    ('TM_DPU_PID6_TC_EXE_NOT_EXECUTABLE', 'RPW'),
    ('TM_DPU_PID6_TC_EXE_NOT_IMPLEMENTED', 'RPW'),
    ('TM_DPU_PID6_TC_EXE_SUCCESS', 'RPW'),
    ('TM_DPU_RWF_HK', 'RPW'),
    ('TM_DPU_S20_HK', 'RPW'),
    ('TM_DPU_SCIENCE_BIAS_CALIB', 'RPW'),
    ('TM_DPU_SCIENCE_BIAS_CALIB_C', 'RPW'),
    ('TM_DPU_SCIENCE_BIAS_CALIB_LONG', 'RPW'),
    ('TM_DPU_SCIENCE_BIAS_CALIB_LONG_C', 'RPW'),
    ('TM_DPU_TC_ACC_ILLEGAL_APID', 'RPW'),
    ('TM_DPU_TC_ACC_ILLEGAL_SUBTYPE', 'RPW'),
    ('TM_DPU_TC_ACC_ILLEGAL_TYPE', 'RPW'),
    ('TM_DPU_TC_ACC_INCOMPLETE', 'RPW'),
    ('TM_DPU_TC_ACC_NOT_ALLOWED', 'RPW'),
    ('TM_DPU_TC_ACC_SUCCESS', 'RPW'),
    ('TM_DPU_TC_ACC_WRONG_CRC', 'RPW'),
    ('TM_DPU_TC_ACC_WRONG_SOURCE_ID', 'RPW'),
    ('TM_DPU_TC_EXE_ERROR', 'RPW'),
    ('TM_DPU_TC_EXE_INCONSISTENT', 'RPW'),
    ('TM_DPU_TC_EXE_NOT_EXECUTABLE', 'RPW'),
    ('TM_DPU_TC_EXE_NOT_IMPLEMENTED', 'RPW'),
    ('TM_DPU_TC_EXE_SUCCESS', 'RPW'),
    ('TM_LFR_HK', 'RPW'),
    ('TM_LFR_KCOEFFICIENTS_DUMP', 'RPW'),
    ('TM_LFR_MEMORY_CHECK', 'RPW'),
    ('TM_LFR_MEMORY_DUMP_RAM', 'RPW'),
    ('TM_LFR_MEMORY_DUMP_REGISTER', 'RPW'),
    ('TM_LFR_PARAMETER_DUMP', 'RPW'),
    ('TM_LFR_SCIENCE_BURST_BP1_F0', 'RPW'),
    ('TM_LFR_SCIENCE_BURST_BP1_F1', 'RPW'),
    ('TM_LFR_SCIENCE_BURST_BP2_F0', 'RPW'),
    ('TM_LFR_SCIENCE_BURST_BP2_F1', 'RPW'),
    ('TM_LFR_SCIENCE_BURST_CWF_F2', 'RPW'),
    ('TM_LFR_SCIENCE_BURST_CWF_F2_C', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_ASM_F0', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_ASM_F1', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_ASM_F2', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_BP1_F0', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_BP1_F1', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_BP1_F2', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_BP2_F0', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_BP2_F1', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_BP2_F2', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_CWF_F3', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_CWF_F3_C', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_CWF_LONG_F3', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_CWF_LONG_F3_C', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_SWF_F0', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_SWF_F0_C', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_SWF_F1', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_SWF_F1_C', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_SWF_F2', 'RPW'),
    ('TM_LFR_SCIENCE_NORMAL_SWF_F2_C', 'RPW'),
    ('TM_LFR_SCIENCE_SBM1_BP1_F0', 'RPW'),
    ('TM_LFR_SCIENCE_SBM1_BP2_F0', 'RPW'),
    ('TM_LFR_SCIENCE_SBM1_CWF_F1', 'RPW'),
    ('TM_LFR_SCIENCE_SBM1_CWF_F1_C', 'RPW'),
    ('TM_LFR_SCIENCE_SBM2_BP1_F0', 'RPW'),
    ('TM_LFR_SCIENCE_SBM2_BP1_F1', 'RPW'),
    ('TM_LFR_SCIENCE_SBM2_BP2_F0', 'RPW'),
    ('TM_LFR_SCIENCE_SBM2_BP2_F1', 'RPW'),
    ('TM_LFR_SCIENCE_SBM2_CWF_F2', 'RPW'),
    ('TM_LFR_SCIENCE_SBM2_CWF_F2_C', 'RPW'),
    ('TM_LFR_TC_ACC_ILLEGAL_APID', 'RPW'),
    ('TM_LFR_TC_ACC_ILLEGAL_SUBTYPE', 'RPW'),
    ('TM_LFR_TC_ACC_ILLEGAL_TYPE', 'RPW'),
    ('TM_LFR_TC_ACC_INCOMPLETE', 'RPW'),
    ('TM_LFR_TC_ACC_NOT_ALLOWED', 'RPW'),
    ('TM_LFR_TC_ACC_SUCCESS', 'RPW'),
    ('TM_LFR_TC_ACC_WRONG_CRC', 'RPW'),
    ('TM_LFR_TC_ACC_WRONG_SOURCE_ID', 'RPW'),
    ('TM_LFR_TC_EXE_CORRUPTED', 'RPW'),
    ('TM_LFR_TC_EXE_ERROR', 'RPW'),
    ('TM_LFR_TC_EXE_INCONSISTENT', 'RPW'),
    ('TM_LFR_TC_EXE_NOT_EXECUTABLE', 'RPW'),
    ('TM_LFR_TC_EXE_NOT_IMPLEMENTED', 'RPW'),
    ('TM_LFR_TC_EXE_SUCCESS', 'RPW'),
    ('TM_RPW_MEMORY_CHECK', 'RPW'),
    ('TM_RPW_MEMORY_DUMP', 'RPW'),
    ('TM_RPW_TC_ACC_ILLEGAL_APID', 'RPW'),
    ('TM_RPW_TC_ACC_ILLEGAL_SUBTYPE', 'RPW'),
    ('TM_RPW_TC_ACC_ILLEGAL_TYPE', 'RPW'),
    ('TM_RPW_TC_ACC_INCOMPLETE', 'RPW'),
    ('TM_RPW_TC_ACC_NOT_ALLOWED', 'RPW'),
    ('TM_RPW_TC_ACC_SUCCESS', 'RPW'),
    ('TM_RPW_TC_ACC_WRONG_CRC', 'RPW'),
    ('TM_RPW_TC_ACC_WRONG_SOURCE_ID', 'RPW'),
    ('TM_RPW_TC_EXE_CORRUPTED', 'RPW'),
    ('TM_RPW_TC_EXE_ERROR', 'RPW'),
    ('TM_RPW_TC_EXE_INCONSISTENT', 'RPW'),
    ('TM_RPW_TC_EXE_NOT_EXECUTABLE', 'RPW'),
    ('TM_RPW_TC_EXE_NOT_IMPLEMENTED', 'RPW'),
    ('TM_RPW_TC_EXE_SUCCESS', 'RPW'),
    ('TM_TDS_DIAGNOSIS', 'RPW'),
    ('TM_TDS_HK', 'RPW'),
    ('TM_TDS_MEMORY_CHECK', 'RPW'),
    ('TM_TDS_MEMORY_DUMP_RAM', 'RPW'),
    ('TM_TDS_MEMORY_DUMP_REGISTER', 'RPW'),
    ('TM_TDS_PARAMETER_DUMP', 'RPW'),
    ('TM_TDS_SCIENCE_LFM_CWF', 'RPW'),
    ('TM_TDS_SCIENCE_LFM_CWF_C', 'RPW'),
    ('TM_TDS_SCIENCE_LFM_PSD', 'RPW'),
    ('TM_TDS_SCIENCE_LFM_RSWF', 'RPW'),
    ('TM_TDS_SCIENCE_LFM_RSWF_C', 'RPW'),
    ('TM_TDS_SCIENCE_LFM_SM', 'RPW'),
    ('TM_TDS_SCIENCE_NORMAL_HIST1D', 'RPW'),
    ('TM_TDS_SCIENCE_NORMAL_HIST2D', 'RPW'),
    ('TM_TDS_SCIENCE_NORMAL_MAMP', 'RPW'),
    ('TM_TDS_SCIENCE_NORMAL_MAMP_C', 'RPW'),
    ('TM_TDS_SCIENCE_NORMAL_RSWF', 'RPW'),
    ('TM_TDS_SCIENCE_NORMAL_RSWF_C', 'RPW'),
    ('TM_TDS_SCIENCE_NORMAL_STAT', 'RPW'),
    ('TM_TDS_SCIENCE_NORMAL_TSWF', 'RPW'),
    ('TM_TDS_SCIENCE_NORMAL_TSWF_C', 'RPW'),
    ('TM_TDS_SCIENCE_SBM1_RSWF', 'RPW'),
    ('TM_TDS_SCIENCE_SBM1_RSWF_C', 'RPW'),
    ('TM_TDS_SCIENCE_SBM2_TSWF', 'RPW'),
    ('TM_TDS_SCIENCE_SBM2_TSWF_C', 'RPW'),
    ('TM_TDS_TC_ACC_ILLEGAL_APID', 'RPW'),
    ('TM_TDS_TC_ACC_ILLEGAL_SUBTYPE', 'RPW'),
    ('TM_TDS_TC_ACC_ILLEGAL_TYPE', 'RPW'),
    ('TM_TDS_TC_ACC_INCOMPLETE', 'RPW'),
    ('TM_TDS_TC_ACC_NOT_ALLOWED', 'RPW'),
    ('TM_TDS_TC_ACC_SUCCESS', 'RPW'),
    ('TM_TDS_TC_ACC_WRONG_CRC', 'RPW'),
    ('TM_TDS_TC_ACC_WRONG_SOURCE_ID', 'RPW'),
    ('TM_TDS_TC_EXE_CORRUPTED', 'RPW'),
    ('TM_TDS_TC_EXE_ERROR', 'RPW'),
    ('TM_TDS_TC_EXE_INCONSISTENT', 'RPW'),
    ('TM_TDS_TC_EXE_NOT_EXECUTABLE', 'RPW'),
    ('TM_TDS_TC_EXE_NOT_IMPLEMENTED', 'RPW'),
    ('TM_TDS_TC_EXE_SUCCESS', 'RPW'),
    ('TM_THR_HK', 'RPW'),
    ('TM_THR_MEMORY_CHECK', 'RPW'),
    ('TM_THR_MEMORY_DUMP_RAM', 'RPW'),
    ('TM_THR_MEMORY_DUMP_REGISTER', 'RPW'),
    ('TM_THR_PARAMETER_DUMP', 'RPW'),
    ('TM_THR_SCIENCE_BURST_HFR', 'RPW'),
    ('TM_THR_SCIENCE_BURST_TNR', 'RPW'),
    ('TM_THR_SCIENCE_CALIBRATION_HFR', 'RPW'),
    ('TM_THR_SCIENCE_CALIBRATION_TNR', 'RPW'),
    ('TM_THR_SCIENCE_NORMAL_HFR', 'RPW'),
    ('TM_THR_SCIENCE_NORMAL_TNR', 'RPW'),
    ('TM_THR_SCIENCE_SPECTRAL_POWER', 'RPW'),
    ('TM_THR_TC_ACC_ILLEGAL_APID', 'RPW'),
    ('TM_THR_TC_ACC_ILLEGAL_SUBTYPE', 'RPW'),
    ('TM_THR_TC_ACC_ILLEGAL_TYPE', 'RPW'),
    ('TM_THR_TC_ACC_INCOMPLETE', 'RPW'),
    ('TM_THR_TC_ACC_NOT_ALLOWED', 'RPW'),
    ('TM_THR_TC_ACC_SUCCESS', 'RPW'),
    ('TM_THR_TC_ACC_WRONG_CRC', 'RPW'),
    ('TM_THR_TC_ACC_WRONG_SOURCE_ID', 'RPW'),
    ('TM_THR_TC_EXE_CORRUPTED', 'RPW'),
    ('TM_THR_TC_EXE_ERROR', 'RPW'),
    ('TM_THR_TC_EXE_INCONSISTENT', 'RPW'),
    ('TM_THR_TC_EXE_NOT_EXECUTABLE', 'RPW'),
    ('TM_THR_TC_EXE_NOT_IMPLEMENTED', 'RPW'),
    ('TM_THR_TC_EXE_SUCCESS', 'RPW');


-- FROM XAVIER
INSERT INTO pipeline.events (label, origin) VALUES
    ('BIA_SWEEP_ANT1', 'RPW'),
    ('BIA_SWEEP_ANT2', 'RPW'),
    ('BIA_SWEEP_ANT3', 'RPW'),
    ('LFR CALIBRATION', 'RPW'),
    ('SBM1 EVENT', 'RPW'),
    ('SBM2 EVENT', 'RPW');

-- UNTRACK SOME EVENTS

UPDATE pipeline.events
    SET is_tracked = false
    WHERE label IN ('TM_DPU_EVENT_LE_DPU_AHB', 'TC_TDS_DUMP_NORMAL_TSWF');

-- SET ANOMALIES

UPDATE pipeline.events
    SET is_anomaly = true
    WHERE label IN (
        'TM_DPU_TC_ACC_ILLEGAL_APID',
        'TM_DPU_TC_ACC_INCOMPLETE',
        'TM_DPU_TC_ACC_WRONG_CRC',
        'TM_DPU_TC_ACC_ILLEGAL_TYPE',
        'TM_DPU_TC_ACC_ILLEGAL_SUBTYPE',
        'TM_DPU_TC_ACC_NOT_ALLOWED',
        'TM_DPU_TC_ACC_WRONG_SOURCE_ID',
        'TM_DPU_TC_EXE_INCONSISTENT',
        'TM_DPU_TC_EXE_NOT_EXECUTABLE',
        'TM_DPU_TC_EXE_NOT_IMPLEMENTED',
        'TM_DPU_TC_EXE_ERROR',
        'TM_DPU_OBC_TC_ACC_ILLEGAL_APID',
        'TM_DPU_OBC_TC_ACC_INCOMPLETE',
        'TM_DPU_OBC_TC_ACC_WRONG_CRC',
        'TM_DPU_OBC_TC_ACC_ILLEGAL_TYPE',
        'TM_DPU_OBC_TC_ACC_ILLEGAL_SUBTYPE',
        'TM_DPU_OBC_TC_ACC_NOT_ALLOWED',
        'TM_DPU_OBC_TC_ACC_WRONG_SOURCE_ID',
        'TM_DPU_OBC_TC_EXE_INCONSISTENT',
        'TM_DPU_OBC_TC_EXE_NOT_EXECUTABLE',
        'TM_DPU_OBC_TC_EXE_NOT_IMPLEMENTED',
        'TM_DPU_OBC_TC_EXE_ERROR',
        'TM_DPU_EVENT_LE_DPU_ADC',
        'TM_DPU_EVENT_LE_DPU_PDU_LINK',
        'TM_DPU_EVENT_LE_DPU_SC_LINK',
        'TM_DPU_EVENT_LE_DPU_PDU_SIS',
        'TM_DPU_EVENT_LE_DPU_AHB',
        'TM_DPU_EVENT_LE_DPU_SC_SPW',
        'TM_DPU_EVENT_LE_DPU_TIME',
        'TM_DPU_EVENT_LE_DPU_TIMECODE',
        'TM_DPU_EVENT_LE_DPU_RESET',
        'TM_DPU_EVENT_ME_DPU_AHB',
        'TM_DPU_EVENT_ME_DPU_EEPROM',
        'TM_DPU_EVENT_ME_DPU_MEMORY',
        'TM_DPU_EVENT_ME_DPU_PDU_SIS',
        'TM_DPU_EVENT_ME_DPU_SC_SPW',
        'TM_DPU_EVENT_ME_DPU_BOOT',
        'TM_DPU_EVENT_ME_DBS_BUFFER',
        'TM_DPU_EVENT_ME_DPU_PARAM',
        'TM_DPU_EVENT_HE_DPU_ADC',
        'TM_DPU_EVENT_HE_DPU_PDU_LINK',
        'TM_DPU_EVENT_HE_DPU_RESET',
        'TM_DPU_PID5_TC_ACC_INCOMPLETE',
        'TM_DPU_PID5_TC_ACC_WRONG_CRC',
        'TM_DPU_PID5_TC_ACC_ILLEGAL_TYPE',
        'TM_DPU_PID5_TC_ACC_ILLEGAL_SUBTYPE',
        'TM_DPU_PID5_TC_ACC_NOT_ALLOWED',
        'TM_DPU_PID5_TC_ACC_WRONG_SOURCE_ID',
        'TM_DPU_PID5_TC_ACC_ILLEGAL_APID',
        'TM_DPU_PID5_TC_EXE_INCONSISTENT',
        'TM_DPU_PID5_TC_EXE_NOT_EXECUTABLE',
        'TM_DPU_PID5_TC_EXE_NOT_IMPLEMENTED',
        'TM_DPU_PID5_TC_EXE_ERROR',
        'TM_DPU_PID6_TC_ACC_INCOMPLETE',
        'TM_DPU_PID6_TC_ACC_WRONG_CRC',
        'TM_DPU_PID6_TC_ACC_ILLEGAL_TYPE',
        'TM_DPU_PID6_TC_ACC_ILLEGAL_SUBTYPE',
        'TM_DPU_PID6_TC_ACC_NOT_ALLOWED',
        'TM_DPU_PID6_TC_ACC_WRONG_SOURCE_ID',
        'TM_DPU_PID6_TC_ACC_ILLEGAL_APID',
        'TM_DPU_PID6_TC_EXE_INCONSISTENT',
        'TM_DPU_PID6_TC_EXE_NOT_EXECUTABLE',
        'TM_DPU_PID6_TC_EXE_NOT_IMPLEMENTED',
        'TM_DPU_PID6_TC_EXE_ERROR',
        'TM_DPU_EVENT_LE_DPU_LFR_SPW',
        'TM_DPU_EVENT_LE_DPU_TDS_SPW',
        'TM_DPU_EVENT_LE_DPU_THR_SPW',
        'TM_DPU_EVENT_LE_DPU_LFR_RMAP',
        'TM_DPU_EVENT_LE_DPU_TDS_RMAP',
        'TM_DPU_EVENT_LE_DPU_THR_RMAP',
        'TM_DPU_EVENT_LE_RPW_CURRENT',
        'TM_DPU_EVENT_LE_DPU_BIA_SIS',
        'TM_DPU_EVENT_LE_DPU_BIA_LINK',
        'TM_DPU_EVENT_LE_DPU_COMP',
        'TM_DPU_EVENT_LE_DPU_LFR_LINK',
        'TM_DPU_EVENT_LE_DPU_TDS_LINK',
        'TM_DPU_EVENT_LE_DPU_THR_LINK',
        'TM_DPU_EVENT_ME_DPU_LFR_SPW',
        'TM_DPU_EVENT_ME_DPU_TDS_SPW',
        'TM_DPU_EVENT_ME_DPU_THR_SPW',
        'TM_DPU_EVENT_ME_DPU_LFR_RMAP',
        'TM_DPU_EVENT_ME_DPU_TDS_RMAP',
        'TM_DPU_EVENT_ME_DPU_THR_RMAP',
        'TM_DPU_EVENT_ME_RPW_CURRENT',
        'TM_DPU_EVENT_ME_LFR_BOOT',
        'TM_DPU_EVENT_ME_TDS_BOOT',
        'TM_DPU_EVENT_ME_THR_BOOT',
        'TM_DPU_EVENT_ME_DPU_BIA_SIS',
        'TM_DPU_EVENT_ME_DAS_BUFFER',
        'TM_DPU_EVENT_ME_LFR_PROCESSOR',
        'TM_DPU_EVENT_ME_TDS_PROCESSOR',
        'TM_DPU_EVENT_ME_THR_PROCESSOR',
        'TM_DPU_EVENT_ME_LFR_COMMAND',
        'TM_DPU_EVENT_ME_TDS_COMMAND',
        'TM_DPU_EVENT_ME_THR_COMMAND',
        'TM_DPU_EVENT_ME_DPU_MODE',
        'TM_DPU_EVENT_ME_LFR_MODE',
        'TM_DPU_EVENT_ME_TDS_MODE',
        'TM_DPU_EVENT_ME_THR_MODE',
        'TM_DPU_EVENT_ME_PDU_OPERATION',
        'TM_DPU_EVENT_ME_DPU_TIME',
        'TM_DPU_EVENT_ME_DPU_SC_LINK',
        'TM_DPU_EVENT_ME_BIA_SWEEP',
        'TM_DPU_EVENT_ME_BIA_CALIB',
        'TM_DPU_EVENT_ME_DPU_WAIT_SYNC',
        'TM_DPU_EVENT_HE_RPW_TEMPERATURE',
        'TM_DPU_EVENT_HE_DPU_BIA_LINK',
        'TM_DPU_EVENT_HE_DPU_LFR_LINK',
        'TM_DPU_EVENT_HE_DPU_TDS_LINK',
        'TM_DPU_EVENT_HE_DPU_THR_LINK',
        'TM_DPU_EVENT_HE_BIAS_HV',
        'TM_DPU_EVENT_HE_RPW_OVC',
        'TM_DPU_EVENT_HE_RPW_OVV',
        'TM_LFR_TC_ACC_INCOMPLETE',
        'TM_LFR_TC_ACC_WRONG_CRC',
        'TM_LFR_TC_ACC_ILLEGAL_TYPE',
        'TM_LFR_TC_ACC_ILLEGAL_SUBTYPE',
        'TM_LFR_TC_ACC_NOT_ALLOWED',
        'TM_LFR_TC_ACC_WRONG_SOURCE_ID',
        'TM_LFR_TC_ACC_ILLEGAL_APID',
        'TM_LFR_TC_EXE_INCONSISTENT',
        'TM_LFR_TC_EXE_NOT_EXECUTABLE',
        'TM_LFR_TC_EXE_NOT_IMPLEMENTED',
        'TM_LFR_TC_EXE_ERROR',
        'TM_LFR_TC_EXE_CORRUPTED',
        'TM_TDS_TC_ACC_INCOMPLETE',
        'TM_TDS_TC_ACC_WRONG_CRC',
        'TM_TDS_TC_ACC_ILLEGAL_TYPE',
        'TM_TDS_TC_ACC_ILLEGAL_SUBTYPE',
        'TM_TDS_TC_ACC_NOT_ALLOWED',
        'TM_TDS_TC_ACC_WRONG_SOURCE_ID',
        'TM_TDS_TC_ACC_ILLEGAL_APID',
        'TM_TDS_TC_EXE_INCONSISTENT',
        'TM_TDS_TC_EXE_NOT_EXECUTABLE',
        'TM_TDS_TC_EXE_NOT_IMPLEMENTED',
        'TM_TDS_TC_EXE_ERROR',
        'TM_TDS_TC_EXE_CORRUPTED',
        'TM_THR_TC_ACC_INCOMPLETE',
        'TM_THR_TC_ACC_WRONG_CRC',
        'TM_THR_TC_ACC_ILLEGAL_TYPE',
        'TM_THR_TC_ACC_ILLEGAL_SUBTYPE',
        'TM_THR_TC_ACC_NOT_ALLOWED',
        'TM_THR_TC_ACC_WRONG_SOURCE_ID',
        'TM_THR_TC_ACC_ILLEGAL_APID',
        'TM_THR_TC_EXE_INCONSISTENT',
        'TM_THR_TC_EXE_NOT_EXECUTABLE',
        'TM_THR_TC_EXE_NOT_IMPLEMENTED',
        'TM_THR_TC_EXE_ERROR',
        'TM_THR_TC_EXE_CORRUPTED',
        'TM_RPW_TC_ACC_ILLEGAL_APID',
        'TM_RPW_TC_ACC_INCOMPLETE',
        'TM_RPW_TC_ACC_WRONG_CRC',
        'TM_RPW_TC_ACC_ILLEGAL_TYPE',
        'TM_RPW_TC_ACC_ILLEGAL_SUBTYPE',
        'TM_RPW_TC_ACC_NOT_ALLOWED',
        'TM_RPW_TC_ACC_WRONG_SOURCE_ID',
        'TM_RPW_TC_EXE_INCONSISTENT',
        'TM_RPW_TC_EXE_NOT_EXECUTABLE',
        'TM_RPW_TC_EXE_NOT_IMPLEMENTED',
        'TM_RPW_TC_EXE_ERROR',
        'TM_RPW_TC_EXE_CORRUPTED'
    );
